function getVars() {
    var vars = [], hash;
    var q = document.URL.split('?')[1];
    if (q != undefined) {
        q = q.split('&');
        for (var i = 0; i < q.length; i++) {
            hash = q[i].split('=');
            vars.push(hash[1]);
            vars[hash[0]] = hash[1];
        }
    }
    return vars;
}

var skillsToAbility = {
    "Acrobatics": "dex",
    "Animal Handling": "wis",
    "Arcana": "int",
    "Athletics": "str",
    "Deception": "cha",
    "History": "int",
    "Insight": "wis",
    "Intimidation": "cha",
    "Investigation": "int",
    "Medicine": "wis",
    "Nature": "int",
    "Perception": "wis",
    "Performance": "cha",
    "Persuasion": "cha",
    "Religion": "int",
    "Sleight of Hand": "dex",
    "Stealth": "dex",
    "Survival": "wis"
};

// Main stat to modifier mapping
function computeAbilityModifier(abilityScore) {
    return Math.floor((abilityScore - 10) / 2);
}

function formatModifier(modifier) {
    if (modifier >= 0) {
        modifier = "+" + modifier;
    }
    return modifier;
}

function mapStats(characterJson, abilityToModifierStore, proficiencyModifier) {
    var mainStats = ["str", "dex", "con", "int", "wis", "cha"];
    $.each(mainStats, function (i, ability) {
        var abilityScoreValue = characterJson.abilityscores[ability];
        // Set text and add fancy hover interaction
        var abilityTarget = $("#" + ability);
        abilityTarget.text(abilityScoreValue).hover(
            function () {
                $("." + ability).css("color", "red")
            },
            function () {
                $("." + ability).css("color", "")
            });

        // Show the modifier that is a function of the ability score
        var modifier = computeAbilityModifier(abilityScoreValue);

        // Save the result in a possibly provided object
        if (abilityToModifierStore !== undefined) {
            abilityToModifierStore[ability] = modifier;
        }

        abilityTarget.parent().find(".mainStatModifier").first().text(formatModifier(modifier));

        // Deal with the saving throws and proficiencies
        if ($.inArray(ability, characterJson.savingthrow_proficiencies) > -1) {
            modifier += proficiencyModifier;
            $("#save_" + ability).addClass("proficient");
        }
        $("#save_" + ability + " .checkModifier").text(formatModifier(modifier));
    });
}

function computeProficiencyModifier(totalLevels) {
    return (Math.floor((totalLevels - 1) / 4) + 2);
}

function mapSkills(characterJson, abilityToModifierStore, proficiencyModifier) {
    var targetContainer = $("#skills").find("div").first();
    $.each(skillsToAbility, function (skill, ability) {
        var modifier = abilityToModifierStore[ability];
        var proficiencyTag = '';
        if ($.inArray(skill.toLowerCase(), characterJson.skill_proficiences) > -1) {
            proficiencyTag = ' proficient';
            modifier += proficiencyModifier;
        }
        targetContainer.append('<div class="center-block col-xs-6 col-sm-3 col-md-6 ' + ability + proficiencyTag + '">' +
        '<span class="checkModifier">' + formatModifier(modifier) + "</span> " + skill + '<i class="hidden-sm hidden-xs"> (' + ability + ')</i></div>');
    });
}

function mapAttacks(characterJson, abilityToModifierStore, proficiencyModifier) {
    $.each(characterJson.attacks, function (attackName, attackObject) {
        var abilityModifier = abilityToModifierStore[attackObject.ability];
        var attackBonus = abilityModifier;
        if (attackObject.proficient) attackBonus += proficiencyModifier;
        if ($.isNumeric(attackObject.extraAttackMod)) attackBonus += attackObject.extraAttackMod;
        attackBonus = formatModifier(attackBonus);
        var damageBonus = abilityModifier;
        if ($.isNumeric(attackObject.extraDmgMod)) damageBonus += attackObject.extraDmgMod;
        var dmgDieWithModifier = attackObject.die + " " + formatModifier(damageBonus);
        $("#attacksAndSpellcasting").find("table").append(
            "<tr><td>" + attackName + "</td><td>" + attackBonus + "</td><td>" + dmgDieWithModifier + "</td><td>" + attackObject.type + "</td></tr>");
    });
}

function renderListItem(target, item) {
    if (item.indexOf(":") > -1) {
        var arr = item.split(":");
        target.append('<span class="fieldHeader">' + arr[0] +
        '</span><br/><span class="fieldFooter">' + arr[1] + '</span><br/>');
    }
    else {
        target.append(item + "<br/>");
    }
}

function mapEquipment(equipment) {
    $.each(equipment.coins, function (coinType, amount) {
        $("#" + coinType).text(amount);
    });
    var target = $("#equipmentList");
    var sortedEquipment = equipment.other.sort();
    var attuned = $.grep(sortedEquipment, function (item) {
        return item.toLowerCase().indexOf("attuned") > -1;
    });

    $.each(attuned, function (i, item) {
        renderListItem(target, item);
    });

    sortedEquipment = $.grep(sortedEquipment, function (item) {
        return attuned.indexOf(item) < 0;
    });

    $.each(sortedEquipment, function (i, item) {
        renderListItem(target, item);
    });
}

function mapPersonality(personality) {
    $("#personalityTraits").prepend(personality.traits);
    $("#ideals").prepend(personality.ideals);
    $("#bonds").prepend(personality.bonds);
    $("#flaws").prepend(personality.flaws);
}

function mapFeatures(features) {
    var target = $("#featuresandtraits");
    target.html('');
    $.each(features.sort(), function (i, feature) {
        renderTrait(target, feature);
    });
    renderAddTraitButton()
}


////////////////////////traits example///////////////////////////////
function renderAddTraitButton() {
    $("#featuresandtraitsadd").html("");
    $("#featuresandtraitsadd").append("<input type='text' id='newtrait'/><button onclick='addTrait()'>Add</button>");
}

function renderTrait(target, item) {
    if (item.name) {
        target.append("<span id='trait" + item.hash + "'>" +
        "<input type='text' id='inputtrait_" + item.hash + "' value='" + item.name + "'/>" +
        "<button onclick='removeTrait(this)'>Del</button>" +
        " <br/></span>");
    }
}

function removeTrait(btn) {
    btn.parentElement.remove()

}

function addTrait() {
    var text = $('#newtrait').val();
    var newTrait = {};
    newTrait.name = text;
    var target = $("#featuresandtraits");
    renderTrait(target, newTrait)
    $("#newtrait").val('');


}

function collectTraits() {
    var traits = new Array();
    $("input[id*=inputtrait_]").each(function (pos, val) {

        var trait = {};
        var hash = val.id.split("_")[1];

        if (hash != 'undefined') {
            trait.hash = hash;
        }
        trait.name = val.value;
        traits.push(trait);

    });
    return traits;
}

//////////////////////////////skills example/////////////////////////////


function mapSkillsExample(skills) {
    var targetContainer = $("#skills").find("div").first();
    targetContainer.html("");
    $.each(skillsList, function (index, skillTemplate) {
        var skill;
        var modifier = 0;
        var proficiencyTag = false;
        for (var i = 0; i < skills.length; i++) {
            if (skills[i].name == skillTemplate.name) {
                skill = skills[i];
                modifier = skills[i].bonus;
                proficiencyTag = true;
            }
        }
        var append = buildSkillBasicData(skillTemplate.name, skillTemplate.ability, skillTemplate.group, modifier);
        var hash = "no"
        if (skill) {
            append += buildAddedSkilElements(skill.hash, skill.misc, skill.ranks);
            hash = skill.hash;
        } else {
            append += buildAddSkillButton(skillTemplate.name, skillTemplate.ability, skillTemplate.group);
        }


        targetContainer.append(buildSkillDiv(skillTemplate.name, proficiencyTag, append, hash));
    });
}


function buildSkillDiv(skillName, isProficiened, innerHTML, hash) {
    var proficiencyTag = "";
    if (isProficiened) {
        proficiencyTag = ' proficient'
    }
    return '<div class="center-block col-xs-6 col-sm-3 col-md-6 ' + skillName + ' ' + proficiencyTag +
        '" id="' + skillName + '_' + hash + '">' + innerHTML + '</div>';
}

function buildAddSkillButton(skillname, ability, group) {
    var button = "<button onclick='addSkill(\"" + skillname + "\",\"" + ability + "\",\"" + group + "\", this )'>Add</button>";
    return button;
}

function buildAddedSkilElements(hash, misc, ranks) {
    var append = "<input type='text' size='2' id='misc" + hash + "' value='" + misc + "'/>" +
        "<input type='text' size='2' id='rank" + hash + "' value='" + ranks + "'/>" +

        "<button onclick='deleteSkill( this)'>Del</button>";
    return append;
}

function buildSkillBasicData(name, ability, group, modifier) {
    var append = '<span class="checkModifier">' + formatModifier(modifier) + "</span> " +
        '<i class="hidden" id="ability" >' + ability + '</i>' +
        '<i class="hidden" id="group" >' + group + '</i>'
        + name;
    return append;
}

function getSkillBasicData(div) {
    var skill = {};
    skill.ability = $(div).find("#ability").text()
    skill.group = $(div).find("#group").text();
    skill.name = $(div).attr("id").split("_")[0];
    return skill;
}

function collectSkills() {
    var skills = new Array();
    $("#skills").find("div[class*=proficient]").each(function (pos, val) {
        var skill = {};
        var hash = val.id.split("_")[1];
        if (!(hash.indexOf("no", 0) === 0)) {
            skill.hash = hash;
        }
        skill.misc = $("#misc" + hash).val();
        skill.ranks = $("#rank" + hash).val();
        skill.ability = $(val).find("#ability").text()
        skill.group = $(val).find("#group").text();
        skill.name = $(val).attr("id").split("_")[0];
        skills.push(skill);

    });
    return skills;
}

function deleteSkill(btn) {
    var div = btn.parentElement;
    var basic = getSkillBasicData(div)
    var append = buildSkillBasicData(basic.name, basic.ability, basic.group, 0);
    append += buildAddSkillButton(basic.name, basic.ability, basic.group);
    $(div).removeClass(" proficient");
    $(div).html(append)
}

function addSkill(name, ability, group, btn) {
    var tempId = "no" + Math.floor(Math.random() * (1000 - 1) + 1);
    var div = btn.parentElement;
    var append = buildSkillBasicData(name, ability, group, 0);
    append += buildAddedSkilElements(tempId, 0, 0);
    $(div).attr("id", name + "_" + tempId);
    $(div).addClass(" proficient");
    $(div).html(append)
}

/////////////////////////////////////////////////////////////////////////


function mapProficiencesAndLanguages(characterJson) {
    $.each(characterJson.languages.sort().reverse(), function (i, language) {
        $("#otherProficienciesAndLanguages").prepend(language + "<br/>");
    });
    $.each(characterJson.proficiencies.sort().reverse(), function (i, proficiency) {
        $("#otherProficienciesAndLanguages").prepend(proficiency + "<br/>");
    });
}

function mapAppearance(characterJson) {
    $.each(characterJson.appearance, function (attrKey, attrValue) {
        $("#" + attrKey).append(attrValue)
    });
}

function mapContacts(characterJson) {
    $.each(characterJson.contacts, function (i, contact) {
        $("#contactlist").append(contact + "<br/>");
    });
}

function mapSpells(characterJson, abilityToModifierStore, proficiencyModifier) {
    if ("spellcastingAbility" in characterJson.spellcasting) {
        var spellCastingAttackModifier = abilityToModifierStore[characterJson.spellcasting.spellcastingAbility] + proficiencyModifier;
        $("#spellAttackBonus").append("+" + spellCastingAttackModifier);
        $("#spellSaveDC").append("+" + (spellCastingAttackModifier + 8));
        $("#spellcastingAbility").append(characterJson.spellcasting.spellcastingAbility);
    }
    if ("spells" in characterJson.spellcasting) {
        $.each(characterJson.spellcasting.spells, function (attrKey, attrValue) {
            $.each(attrValue, function (spellKey, spellName) {
                $("#" + attrKey).append("<div>" + spellName + "</div>")
            });
        });
        // Remove empty spell slots
        for (var n = 110; n < 10; ++n) {
            var spell = "spells" + n;
            if (!(spell in characterJson.spellcasting.spells)) {
                $("#" + spell).remove();
            }
        }
    }
}

function loadImage(link) {
    $('#charimg').attr("src", link);
}

function loadChar() {

    //loadSkillsList();
    createExampleCombobox();
    loadCharacterDataFromServer();


    document.title = characterJson.name + " - Character Sheet";

    // Compute all classes into a single string, deal with hit dice
    var charClass = [];
    var totalLevels = 0;
    var hitdice = [];
    $.each(characterJson.classes, function (className, classObject) {
        charClass.push(className + " " + classObject.level);
        totalLevels += classObject.level;
        hitdice.push(classObject.hitdice);
    });
    $("#character_class").text(charClass.join(" / "));

    // Proficiency modifier is a function of the total character level
    var proficiencyModifier = computeProficiencyModifier(totalLevels);
    $("#proficiencyModifier").text("+" + proficiencyModifier);

    $("#character_background").html(characterJson.mainattributes.background || "&nbsp");
    $("#character_alignment").html(characterJson.mainattributes.alignment || "&nbsp");
    $("#player_name").html(characterJson.mainattributes.playername || "&nbsp");
    $("#character_xp").html(characterJson.mainattributes.xp || "&nbsp");

    // Map inspiration
    if ((characterJson.hasinspirationpoint || $.isNumeric(characterJson.inspirationpoints) && characterJson.inspirationpoints > 0)) {
        $('#inspiration').prop("checked", true)
    }

    // Map exhaustion
    if (characterJson.exhaustionlevel > 0) {
        $("#exhaustion").find("input[name=exhaustionLevel][value=" + characterJson.exhaustionlevel + "]").prop('checked', true);
    }

    // Map death saves success
    if (characterJson.deathsavesuccess > 0) {
        $("#deathSaveSuccess").find("input[name=deathSaveSuccessLevel][value=" + characterJson.deathsavesuccess + "]").prop('checked', true);
    }

    // Map death saves failure
    if (characterJson.deathsavefailure > 0) {
        $("#deathSaveFailure").find("input[name=deathSaveFailureLevel][value=" + characterJson.deathsavefailure + "]").prop('checked', true);
    }

    // Plug in ability scores, modifiers and saving throws
    var abilityToModifierStore = {};
    mapStats(characterJson, abilityToModifierStore, proficiencyModifier);

    // Handle skills
    // mapSkills(characterJson, abilityToModifierStore, proficiencyModifier);

    // Handle combat stats
    $("#armorClass").prepend(characterJson.ac);
    // If initiative is supplied in the json it will be used, otherwise, the dex modifier is used.
    $("#initiative").prepend(
        formatModifier(characterJson.initiative || abilityToModifierStore["dex"]));

    $("#speed").prepend(characterJson.speed);
    $("#hitpoints").prepend(characterJson.hitpoints);
    $("#hitdice").prepend(hitdice.join(" / "));

    // And for the attacks
    mapAttacks(characterJson, abilityToModifierStore, proficiencyModifier);

    // Coins & other equipment
    mapEquipment(characterJson.equipment);

    mapPersonality(characterJson.personality);
    //mapFeatures(characterJson.features); // now in update ajax
    mapProficiencesAndLanguages(characterJson);
    mapAppearance(characterJson);
    if ("contacts" in characterJson) {
        mapContacts(characterJson);
    }
    if ("spellcasting" in characterJson) {
        mapSpells(characterJson, abilityToModifierStore, proficiencyModifier);
    }
    else {
        $("#spellsContainer").remove();
        $("#spellAbilitiesContainer").remove();
    }
}

var usedPort = window.location.port;
// Inject the JSONp "script" from the location defined in the URL.
var characterJson = {};
var skillsList = {};
var charURL = decodeURIComponent(getVars()["charURL"]);
if (charURL === 'undefined') charURL = 'js/template.js';
loadSkillsList(); // load data
$.ajax({
    url: charURL,
    dataType: 'script',
    timeout: 5000,
    success: function () {
        characterJson = charJson;
        loadChar();
    },
    error: function (XMLHttpRequest, textStatus, errorThrown) {
        alert("Failed loading data from charURL " + charURL + "  - " + textStatus + " - " + errorThrown);
    }
});

// example load creature
function loadCharacterDataFromServer() {
    var uin = getVars()["uin"];
    $.ajax({
        url: 'http://localhost:' + usedPort + '/find/UIN/' + uin,
        type: 'get',
        dataType: 'json',
        contentType: "application/json; charset=utf-8",
        success: function (data) {
            characterJson.uin = getVars()["uin"];
            characterJson.name = data.name;
            characterJson.race = data.type;
            characterJson.traits = data.data.traits;
            characterJson.skills = data.skills;
            characterJson.lastupdate = data.lastupdate;
            $("#character_name").val(characterJson.name);
            $('[id=combo]').val(characterJson.race);
            $("#character_race").html(characterJson.race || "&nbsp");
            mapFeatures(characterJson.traits);
            loadImage(data.imageLink);
            mapSkillsExample(characterJson.skills);
            loadImage(data.imageLink);
        }
    });
}

//load combobox data from server
function createExampleCombobox() {
    $.ajax({
        url: 'http://localhost:' + usedPort + '/getRaceDropdownData',
        type: 'get',
        dataType: 'json',
        contentType: "application/json; charset=utf-8",
        success: function (data) {
            addCombo(data);
        }
    });
}

function loadSkillsList() {
    $.ajax({
        url: 'http://localhost:' + usedPort + '/getSkillsData',
        type: 'get',
        dataType: 'json',
        contentType: "application/json; charset=utf-8",
        success: function (data) {
            skillsList = data;
        }
    });

}

function uploadFile() {

    var file = $('#fileupload').get(0).files[0];

    var uri = 'http://localhost:' + usedPort + '/images/upload';
    var xhr = new XMLHttpRequest();

    var fd = new FormData();
    fd.append('module', $("#module_name").val());
    fd.append('UIN', characterJson.uin);
    fd.append('file', file);


    xhr.open("POST", uri, true);
    xhr.onreadystatechange = function () {
        if (xhr.readyState == 4 && xhr.status == 200) {
            // Handle response.
            alert(xhr.responseText); // handle response.
        }
    };

    // Initiate a multipart/form-data upload
    xhr.send(fd);

}

function addCombo(selects) {
    var combo = $("#combo");
    $.each(selects, function (i, item) {
        $('#combo').append($('<option>', {
            value: item,
            text: item
        }));
    });
}

function collectDataAndSend() {
    var sendData = {};
    sendData.data = {};
    sendData.name = $("#character_name").val();
    sendData.type = $("#combo").find('option:selected').val();
    sendData.data.traits = collectTraits();
    sendData.skills = collectSkills();
    sendDataToREST(sendData)
}

function sendDataToREST(sendData) {
    $.ajax({
        url: 'http://localhost:' + usedPort + '/update/' + characterJson.uin,
        type: 'post',
        dataType: 'json',
        contentType: "application/json; charset=utf-8",
        data: JSON.stringify(sendData),
        success: function (data) {

        }
    });
}

function checkUpdates() {
    $.ajax({
        url: 'http://localhost:' + usedPort + '/update/check/' + characterJson.uin,
        type: 'get',
        dataType: 'json',
        contentType: "application/json; charset=utf-8",
        data: {lastupdate: characterJson.lastupdate},
        success: function (data) {
            $('#chreckUpdateResult').text(data)
        }
    });
}

function selectImage(){
    $.ajax({
        url: 'http://localhost:' + usedPort + '/images',
        type: 'get',
        dataType: 'json',
        contentType: "application/json; charset=utf-8",
        success: function (data) {
           createSelectImageTable(data);
        }
    });
}

function createSelectImageTable(data){
    $('#select_image').show();
    var table = $('#imagesSelect');
    table.innerHTML="";
    for(var i in data)
    {
        var id = data[i].id;
        var link = data[i].link;
        table.append('<tr><td><button onclick="updateImage('+id.toString()+')">Select</button></td><td><img src="'+link+'"  style="width:100px;height:100px;" ></td></tr>');
    }
}

function updateImage(id){
    var data = {
        imageID : id
    }
    sendDataToREST(data)
    $('#select_image').hide();
}

